package fmon.battle

import javafx.fxml.FXML
import javafx.fxml.FXMLLoader
import javafx.fxml.JavaFXBuilderFactory
import javafx.scene.{control => jfxsc}

import scalafx.Includes._
import scalafx.scene.Parent
import scalafx.scene.control._
import scalafx.scene.layout._

import fmon.battle.msg._
import fmon.stat._
import fmon.stat.Statistic._

class BattleUI extends SignalConsumer {
  @FXML var buttonPane: javafx.scene.layout.VBox = _
  @FXML var messages: jfxsc.TextArea = _
  
  @FXML var playerMonController: InfoWidget = _
  @FXML var enemyMonController: InfoWidget = _
  
  private var engine: BattleEngine = _
  
  def setEngine(engine: BattleEngine): Unit = {
    this.engine = engine
    updateUI()
  }
  
  def updateUI(): Unit = {
    val player = engine.player.lead
    val enemy = engine.enemy.lead
    val buttons = player.base.moves.map(move => loadMoveButton(move))
    buttonPane.children = buttons
    
    playerMonController.update(player)
    enemyMonController.update(enemy)
  }
  
  def onMove(move: Move): Unit = {
    val action = Action(engine.player.lead, move, engine.enemy.lead)
    engine.playTurn(action)
    updateUI()
  }
  
  def !(msg: Signal): Unit = {
    msg match {
      case Message(text) => messages.text = s"${messages.text()}${text}\n"
    }
  }
  
  private def loadMoveButton(move: Move): Button = {
    val fxmlLoader = new FXMLLoader(getClass.getResource("moveButton.fxml"))
    fxmlLoader.setBuilderFactory(new JavaFXBuilderFactory())
    val button = fxmlLoader.load[jfxsc.Button]()
    val controller = fxmlLoader.getController[MoveButton]()
    controller.setup(move, onMove)
    button
  }
}